@@ -23,7 +23,7 @@ from flask_caching import Cache
 from fudstop._markets.list_sets.dicts import all_forex_pairs
 from fudstop.apis.webull.webull_parsing import WebullParser
 from fudstop.apis.webull.webull_markets import WebullMarkets
-from fudstop.apis._openai._chat import OpenAISDK
+from fudstop.apis._openai.openai_sdk import OpenAISDK
 from fudstop.apis.fed_print.fed_print_sync import FedPrintSync
 from fudstop.apis.polygonio.polygon_database import PolygonDatabase
 from fudstop.apis.polygonio.polygon_options import PolygonOptions
