@@ -12,7 +12,7 @@ import asyncpg
 from asyncpg.exceptions import UniqueViolationError
 from apis.polygonio.polygon_database import PolygonDatabase
 import aiohttp
-from finished.db_manager import DbManager
+from _markets.finished.db_manager import DbManager
 vwap_diff=os.environ.get('vwap_diff')
 five_1k=os.environ.get('five_1k')
 onek_tenk=os.environ.get('one_tenk')
@@ -20,19 +20,20 @@ tenk_fiftyk=os.environ.get('tenk_fiftyk')
 fiftyk_plus=os.environ.get('fiftyk_plus')
 specials=os.environ.get('specials')
 from apis.webull.webull_trading import WebullTrading as AsyncWebullSDK
+from apis.polygonio.polygon_database import PolygonDatabase
 import anyio
 from apis.helpers import format_large_number
 import httpcore
 from datetime import datetime
 import pytz
-db = PolygonDatabase(user='chuck', database='charlie', host='localhost', port=5432, password='fud')
+
 # Get the current time in UTC
 now_utc = datetime.now(pytz.utc)
 
 # Convert UTC time to Eastern Time
 eastern = pytz.timezone('US/Eastern')
 now_eastern = now_utc.astimezone(eastern)
-
+db = PolygonDatabase(host='localhost', user='chuck', database='markets', password='fud',port=5432)
 # Format the datetime object to the desired format (YYYY-MM-DD HH:MM:SS)
 formatted_time = now_eastern.strftime('%Y-%m-%d %H:%M:%S')
 
@@ -232,7 +233,7 @@ async def send_td9_embed(timespan, hook, ticker, state):
     embed.set_footer(text=f"{ticker} | {timespan} | {state} | data by polygon.io | Implemented by FUDSTOP", icon_url=fudstop)
     webhook.add_embed(embed)
 
-
+    asyncio.create_task()
     asyncio.create_task(webhook.execute())
 
 
@@ -676,7 +677,7 @@ async def specials_embed(buyvol_percent, midvol_percent, underlyingmultivol_perc
 
 
 async def advanced_snapshot_embed_unusual(contract: dict):
-    hook = AsyncDiscordWebhook("https://discord.com/api/webhooks/1165777285383737424/AkEpPcgnTxWUwY4QCSwXaB4Xh43YZH8dddM1_zI0OXDycinJfxsOwZq1UPTFrbBx0_20")
+    hook = AsyncDiscordWebhook(os.environ.get('advanced_snapshot_unusual'))
     volume = contract.get('volume')
     underlying_symbol = contract.get('underlying_symbol')
     sym = contract.get('option_symbol')
@@ -852,91 +853,7 @@ async def advanced_snapshot_embed_oi(contract: dict, hook_url):
 
 
 
-async def conditional_embed(contract: dict, hook_url, title, description):
-    hook = AsyncDiscordWebhook(hook_url)
-    volume = contract.get('volume')
-    underlying_symbol = contract.get('underlying_symbol')
-    sym = contract.get('option_symbol')
-    exp = contract.get('expiry')
-    if exp:
-        exp = exp.split(" ")[0]
-    strike = contract.get('strike')
-    call_put = contract.get('call_put')
-    oi = contract.get('oi')
-    trade_conditions = contract.get('trade_conditions')
-    delta_to_theta = contract.get('delta_to_theta_ratio')
-    change_percent = contract.get('change_percent')
-    iv = contract.get('iv')
-    delta = contract.get('delta')
-    vega = contract.get('vega')
-    theta = contract.get('theta')
-    gamma = contract.get('gamma')
-    name = contract.get('name')
-    open = contract.get('open')
-    high = contract.get('high')
-    low = contract.get('low')
-    under_price = contract.get('underlying_price')
-    close = contract.get('close')
-    prev_close = contract.get('previous_close')
-    time_to_expiry = contract.get('time_to_expiry')
-    trade_size = contract.get('trade_size')
-    trade_price = contract.get('trade_price')
-    timestamp = contract.get('trade_timestamp')
-    cost_of_theta = contract.get('cost_of_theta')
-    implied_leverage = contract.get('implied_leverage')
-    ask = contract.get('ask')
-    bid = contract.get('bid')
-    change = contract.get('change')
-    ask_size = contract.get('ask_size')
-    bid_size = contract.get('bid_size')
-    ask_exchange = contract.get('ask_exchange')
-    bid_exchange = contract.get('bid_exchange')
-    moneyness = contract.get('moneyness')
-    liquidity_score = contract.get('liquidity_score')
-    trade_exchange = contract.get('trade_exchange')
-    extrinsic_value = contract.get('extrinsic_value')
 
-    if gamma is not None and vega is not None and theta is not None and delta is not None:
-        gamma = round(float(gamma),2)
-        delta = round(float(delta),2)
-        vega = round(float(vega),2)
-        theta = round(float(theta),2)
-
-
-    if extrinsic_value is not None:
-        extrinsic_value = round(float(extrinsic_value),2)
-
-    if moneyness is not None:
-        moneyness = round(float(moneyness),2)
-
-
-    if cost_of_theta is not None:
-        cost_of_theta = round(float(cost_of_theta),2)
-
-
-    if implied_leverage is not None:
-        implied_leverage = round(float(implied_leverage),2)
-
-
-    if delta_to_theta is not None:
-        delta_to_theta = round(float(delta_to_theta),2)
-
-
-    if iv is not None:
-        iv = round(float(iv)*100,2)
-    embed = DiscordEmbed(title=title, description=f"```py\nContract: {underlying_symbol} | {strike} | {call_put} | {exp}```\n```py\n{description}.```", color=hex_colors['green'])
-
-    embed.add_embed_field(name=f"Pricing:", value=f"> Underlying: **${under_price}**\n\n> Contract:\n> Open: **${open}**\n> High: **${high}**\n> Low: **${low}**\n> Close: **${close}**\n> Prev. Close: **${prev_close}**\n> Change Percent: **{change_percent}**\n> Change: **{change}**")
-    embed.add_embed_field(name=f"Greeks:", value=f"> Delta: **{delta}**\n> Gamma: **{gamma}**\n> Theta: **{theta}**\n> Vega: **{vega}**\n> IV: **{iv}%**")
-    embed.add_embed_field(name=f"Greek Extras:", value=f"> Cost of Theta: **${cost_of_theta}**\n> Delta/Theta Ratio: **{delta_to_theta}**")
-    embed.add_embed_field(name=f"Last Trade:", value=f"> Size: **{trade_size}**\n> Price: **${trade_price}**\n> Exchange: **{trade_exchange}**\n> Conditions: **{trade_conditions}**\n> Time: **{timestamp}**")
-    embed.add_embed_field(name=f"Last Quote:", value=f"> Ask: **${ask}**\n> Ask Size: **{ask_size}**\n> Ask Exchange: **{ask_exchange}**\n\n> Bid: **${bid}**\n> Bid Size: **{bid_size}**\n> Bid Exchange: **{bid_exchange}**")
-    embed.add_embed_field(name=f"Contract Stats:", value=f"> DTE: **{time_to_expiry}**\n> Extrinsic Value: **{extrinsic_value}**\n> Moneyness: **{moneyness}**\n> ImpliedLeverage: **{implied_leverage}**\n> Liquidity Score: **{liquidity_score}**")
-    embed.add_embed_field(name=f"Volume & OI:", value=f"> Vol: **{volume}**\n> OI: **{oi}**")
-    embed.set_footer(text=f'{sym} Data by Polygon.io | Implemented by FUDSTOP', icon_url=fudstop)
-    embed.set_timestamp()
-    hook.add_embed(embed)
-    await hook.execute()
 
 
 
@@ -1257,3 +1174,48 @@ async def theta_resistant_embed(data:dict):
     await hook.execute()
 
 
+
+
+async def sector_dark_pool_embed(logo, ticker, webhook_url, sector, dollar_cost, conditions, exchange, size, price, volume_indicator, trade_timestamp):
+
+    hook = AsyncDiscordWebhook(webhook_url, content=f"<@375862240601047070>")
+    embed = DiscordEmbed(title=f"SECTOR DARK POOLS - {sector} | {ticker}", description=f"```py\n> Size: **{size}**\n> Price: **${price}**\n> Dollar Cost: **${round(float(dollar_cost),2)}**```\n> **These feeds are SECTOR DARK POOLS - Or Trades occuring out of the FINRA ADF/TRF Facility. Trades with dollar value of $1,000,000 or more will post here out of these exchanges.", color=market.colors.get('gold'))
+    embed.add_embed_field(name=f"Conditions:", value=f"> **{conditions}**")
+    embed.add_embed_field(name=f"Exchange:", value=f"> **{exchange}**")
+    embed.add_embed_field(name=f"Volume Indicator:", value=f"> **{volume_indicator}**")
+    embed.set_thumbnail(logo)
+    embed.set_timestamp()
+    embed.set_footer(text=f'{ticker} | {trade_timestamp}', icon_url=os.environ.get('fudstop_logo'))
+
+    hook.add_embed(embed)
+    asyncio.create_task(hook.execute())
+
+
+
+async def conditional_embed(webhook, row, color, description, status, sector, db):
+    hook = AsyncDiscordWebhook(webhook)
+
+    embed = DiscordEmbed(title=f"{row['ticker']} | ${row['underlying_price']} |{row['strike']} | {row['cp']} | {row['expiry']}", description=f"{description}", color=color)
+    embed.add_embed_field(name=f"Contract Stats:", value=f"> OPEN: **${row['open']}**\n> HIGH: **${row['high']}**\n> LOW: **${row['low']}**\n> CLOSE: **${row['close']}**\n> VWAP: **${row['vwap']}**\n> CHANGE%: **{row['change_percent']}**")
+    embed.add_embed_field(name=f"IV:", value=f"> **{row['iv']}**\n> Percentile: **{row['iv_percentile']}**")
+    embed.add_embed_field(name=f"OI / VOl", value=f"> OI: **{row['oi']}**\n> VOL: **{row['vol']}**\n> RATIO: **{row['vol_oi_ratio']}**")
+    embed.add_embed_field(name=f"Value:", value=f"> Intrinsic: **{row['intrinsic_value']}**\n> Extrinsic: **{row['extrinsic_value']}**\n> Time: **{row['time_value']}**")
+    embed.add_embed_field(name=f"Spread:", value=f"> Bid: **${row['bid']}**\n> Mid: **${row['mid']}**\n> Ask: **{row['ask']}**\n> Spread: **{round(float(row['spread']),2)}**\n> Spread PCT: **{row['spread_pct']}%**")
+    embed.add_embed_field(name="Details:", value=f"> Moneyness: **{row['moneyness']}**\n> Velocity: **{row['velocity']}**\n> Profit Potential: **{row['opp']}**")
+    embed.add_embed_field(name=f"GREEKS:", value=f"> Delta: **{row['delta']}**\n> Delta/Theta Ratio: **{row['delta_theta_ratio']}**\n> Gamma: **{row['gamma']}**\n> Gamma Risk: **{row['gamma_risk']}**\n> Vega: **{row['vega']}**\n> Vega Impact: **{row['vega_impact']}**\n> Theta: **{row['theta']}**\n> Decay Rate: **{row['theta_decay_rate']}**", inline=False)
+    embed.set_timestamp()
+    embed.set_footer(text=f'{type} | Data by Polygon.io | Implemented by FUDSTOP', icon_url=os.environ.get('fudstop_logo'))
+    hook.add_embed(embed)
+    asyncio.create_task(hook.execute())
+
+    data_dict = { 
+        'ticker': row['ticker'],
+        'sector': sector,
+        'description': description,
+        'status': status
+    }
+   
+
+    df = pd.DataFrame(data_dict, index=[0])
+
+    await db.batch_insert_dataframe(df, table_name='feeds', unique_columns='ticker, status, description')
\ No newline at end of file
