@@ -473,28 +473,25 @@ class DbManager:
                     for record in data:
                         new_record = []
                         for col, val in zip(df.columns, record):
-                   
                             pg_type = type_mapping[col]
 
                             if val is None:
                                 new_record.append(None)
-                            elif pg_type == 'timestamp' and isinstance(val, np.datetime64):
-                                new_record.append(pd.Timestamp(val).to_pydatetime().replace(tzinfo=None))
-
-            
-                            elif isinstance(val, datetime):
-                                new_record.append(pd.Timestamp(val).to_pydatetime())
-                            elif pg_type in ['timestamp', 'timestamp without time zone', 'timestamp with time zone'] and isinstance(val, np.datetime64):
-                                new_record.append(pd.Timestamp(val).to_pydatetime().replace(tzinfo=None))  # Modified line
+                            elif pg_type in ['timestamp', 'timestamp without time zone', 'timestamp with time zone']:
+                                if isinstance(val, np.datetime64):
+                                    # Convert numpy datetime64 to Python datetime, ensure UTC and remove tzinfo if needed
+                                    new_record.append(pd.Timestamp(val).to_pydatetime().replace(tzinfo=None))
+                                elif isinstance(val, datetime):
+                                    # Directly use the Python datetime object
+                                    new_record.append(val)
                             elif pg_type in ['double precision', 'real'] and not isinstance(val, str):
                                 new_record.append(float(val))
-                            elif isinstance(val, np.int64):  # Add this line to handle numpy.int64
+                            elif isinstance(val, np.int64):
                                 new_record.append(int(val))
                             elif pg_type == 'integer' and not isinstance(val, int):
                                 new_record.append(int(val))
                             else:
                                 new_record.append(val)
-                    
                         batch_data.append(new_record)
 
                         if len(batch_data) == batch_size:
