@@ -12,10 +12,12 @@ from asyncpg.exceptions import UniqueViolationError
 from aiohttp.client_exceptions import ContentTypeError
 from collections import deque
 from dotenv import load_dotenv
-from embeddings import vol_anal_embed, create_newhigh_embed, profit_ratio_02_embed, profit_ratio_98_embed, option_condition_embed, sized_trade_embed
+from embeddings import vol_anal_embed, create_newhigh_embed, profit_ratio_02_embed, profit_ratio_98_embed, option_condition_embed, sized_trade_embed, conditional_embed
 from _markets.list_sets.dicts import hex_color_dict
 from apis.discord_.discord_sdk import DiscordSDK
+from apis.helpers import identify_sector
 from _markets.list_sets.ticker_lists import most_active_tickers
+from _markets.list_sets.dicts import healthcare, etfs, basic_materials, industrials, communication_services,consumer_cyclical,consumer_defensive,financial_services,energy,real_estate,utilities, technology
 from analyzers import OptionDataAnalyzer
 from market_handlers.database_ import MarketDBManager
 from datetime import datetime, timedelta
@@ -29,6 +31,7 @@ from fudstop.apis.polygonio.mapping import option_condition_desc_dict,option_con
 from list_sets.dicts import all_forex_pairs, crypto_currency_pairs
 from apis.polygonio.technicals import Technicals
 from apis.webull.webull_trading import WebullTrading
+from apis.polygonio.polygon_database import PolygonDatabase
 # Create a reverse dictionary
 all_forex_pairs = {v: k for k, v in all_forex_pairs.items()}
 from typing import List
@@ -45,16 +48,16 @@ class MultiMarkets(Inserts):
     def __init__(self, user, database, port, host, password):
         self.poly = Polygon(host='localhost', user='chuck', database='markets', password='fud', port=5432)
         self.discord = DiscordSDK()
-
+        self.feed_db = PolygonDatabase(host='localhost', user='chuck', port=5432, password='fud', database='markets')
         self.technicals = Technicals()
         self.db = MarketDBManager(user=user,database=database,port=port,host=host,password=password)
-        self.markets = [Market.Stocks, Market.Options] #Market.Forex]# Market.Indices]
+        self.markets = [Market.Stocks, Market.Options,  Market.Forex] #Market.Forex]# Market.Indices]
         self.subscription_patterns = {
             Market.Options: ["T.*,A.*"],
             Market.Stocks: ["A.*,T.*"],
-            #Market.Indices: ["A.*"],
-            #Market.Crypto: ['XT.*, XL2.*'],
-            #Market.Forex: ['CAS.*']
+           # Market.Indices: ["A.*"],
+            #Market.Crypto: ['XT.*'],
+            Market.Forex: ['CAS.*']
 
         }
         self.ticker_cache = {}
@@ -71,7 +74,7 @@ class MultiMarkets(Inserts):
         self.last_ticker = None
         self.consecutive_count = 0
         self.indices_names=indices_names_and_symbols_dict
-        self.agg_tickers = deque(maxlen=250)
+        self.agg_tickers = deque(maxlen=25)
         self.trade_tickers = deque(maxlen=250)
         self.opts = PolygonOptions(user='chuck', database='markets', host='localhost', port=5432, password='fud')
         self.timespans = ['minute', 'hour', 'day', 'week', 'month']
@@ -85,7 +88,7 @@ class MultiMarkets(Inserts):
         self.eight_days_from_now = (datetime.now() + timedelta(days=8)).strftime('%Y-%m-%d')
         self.eight_days_ago = (datetime.now() - timedelta(days=8)).strftime('%Y-%m-%d')    
     # Function to check if the stock should be processed
-    def should_process_stock(self, ticker):
+    async def should_process_stock(self, ticker):
         current_time = time.time()
         if ticker in self.ticker_cache and current_time - self.ticker_cache[ticker] < 60:
             return False
@@ -105,9 +108,8 @@ class MultiMarkets(Inserts):
 
 
     async def stock_rsi(self, ticker):
-   
-        await self.poly.create_session()
-
+        
+        
 
 
         timespan=random.choice(self.timespans)
@@ -118,7 +120,7 @@ class MultiMarkets(Inserts):
         latest_rsi = None
         if rsi is not None and rsi.rsi_value is not None:
             latest_rsi = rsi.rsi_value[0]
-            print(latest_rsi)
+
 
         if timespan == 'hour':
             hook = AsyncDiscordWebhook(os.environ.get('osob_hour'))
@@ -134,18 +136,35 @@ class MultiMarkets(Inserts):
             hook = AsyncDiscordWebhook(os.environ.get('osob_week'))
 
         if timespan == 'month':
-            hook = AsyncDiscordWebhook(os.environ.get('osob_mth'))
+            hook = AsyncDiscordWebhook(os.environ.get('osob_month'))
 
         
         if timespan is not None and hook is not None and latest_rsi is not None:
+
             color = hex_color_dict['green'] if latest_rsi <=30 else hex_color_dict['red'] if latest_rsi >= 70 else hex_color_dict['grey']
-            status = 'oversold' if color == hex_color_dict['green'] else 'overbought' if color == hex_color_dict['red'] else None
+            status = f'oversold {timespan}' if color == hex_color_dict['green'] else f'overbought {timespan}' if color == hex_color_dict['red'] else None
+            
+            description=f"```py\n{ticker} is {status} on the {timespan} with an RSI value of {latest_rsi}.```"
+            
             if status is not None:
+                sector = await identify_sector(ticker)
                 embed = DiscordEmbed(title=f"RSI Results - ALL {timespan}", description=f"```py\n{ticker} is {status} on the {timespan} with an RSI value of {latest_rsi}.```", color=color)
                 embed.set_timestamp()
                 embed.set_footer(f'overbought / oversold RSI feed - {timespan}')
                 hook.add_embed(embed)
-                await hook.execute()
+                asyncio.create_task(hook.execute())
+
+
+                data_dict = { 
+                    'ticker':ticker,
+                    'sector': sector,
+                    'status': status,
+                    'description': description,
+                }
+
+
+                df = pd.DataFrame(data_dict, index=[0])
+                asyncio.create_task(self.db.batch_insert_dataframe(df, 'feeds', 'ticker, status, description'))
 
 
 
@@ -153,15 +172,15 @@ class MultiMarkets(Inserts):
 
 
     async def stock_macd(self, ticker):
+       
+        macd_m= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_minute))
+        macd_d= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_day))
+        macd_h= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_hour))
+        macd_w= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_week))
+        macd_mth= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_month))
         
-       macd_m= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_minute))
-       macd_d= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_day))
-       macd_h= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_hour))
-       macd_w= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_week))
-       macd_mth= asyncio.create_task(self.technicals.run_technical_scanner(ticker, self.time_month))
-    
 
-       await asyncio.gather(macd_m, macd_d, macd_h, macd_w, macd_mth)
+        await asyncio.gather(macd_m, macd_d, macd_h, macd_w, macd_mth)
 
 
 
@@ -269,7 +288,8 @@ class MultiMarkets(Inserts):
         asyncio.create_task(self.db.batch_insert_dataframe(data.as_dataframe, table_name='poly_opts', unique_columns='insertion_timestamp', batch_size=1))
 
         df = data.as_dataframe
-        yield df
+        for i,row in df.iterrows():
+            yield row
 
 
 
@@ -278,7 +298,7 @@ class MultiMarkets(Inserts):
             
     async def handle_msg(self, msgs: WebSocketMessage):
 
-  
+        
         monitor = EquityOptionTradeMonitor()
         
         for m in msgs:
@@ -288,199 +308,110 @@ class MultiMarkets(Inserts):
 
 
             if event_type == 'A' and m.symbol.startswith('O:'):
+                
                 async for data in self.db.insert_option_aggs(m):
                     ticker = data.get('option_symbol')
                     symbol = data.get('ticker')
                     self.agg_tickers.append(m.symbol)  # Append to the instance's deque
 
                     # When we have 250 tickers, process them
-                    if len(self.agg_tickers) == 250:
+                    if len(self.agg_tickers) == 25:
+                        
                         tickers_string = ','.join(self.agg_tickers)
+                        
                         # Process the tickers_string as needed
                     
 
-
-                        async for df in self.process_batches(tickers_string):
-                            processed_data = set()  # Initialize a set to store processed data points
-                            
-                            for i,row in df.head(5).iterrows():
-                                data_identifier = f"{row['ticker']}_{row['timestamp']}"
-
-                                # Check if the data point has already been processed
-                                if data_identifier in processed_data:
-                                    continue  # Skip processing if it's a duplicate
-                                else:
-                                    processed_data.add(data_identifier)
-                                # Access each column value from the row
+                        self.agg_tickers.clear()
+                        try:
+                            async for row in self.process_batches(tickers_string):
+                                processed_data = set()  # Initialize a set to store processed data points
                                 oi_value = row['oi']
-                                vol_value = row['vol']
-                                vol_oi_value = row['vol_oi_ratio']
-                                iv_value = row['iv']
-                                ask_value = row['ask']
-                                bid_value = row['bid']
-                                mid_value = row['mid']
-                                strike_value = row['strike']
-                                expiry_value = row['expiry']
-                                call_put_value = row['cp']
                                 underlying_symbol_value = row['ticker']
                                 underlying_price_value = row['underlying_price']
-                                gamma_value = row['gamma']
-                                gamma_risk_value = row['gamma_risk']
-                                delta_value = row['delta']
-                                delta_theta_value = row['delta_theta_ratio']
-                                theta_value = row['theta']
-                                theta_decay_value = row['theta_decay_rate']
-                                vega_value = row['vega']
-                                vega_impact_value = row['vega_impact']
+                                strike_value = row['strike']
                                 change_percent_value = row['change_percent']
-                                close_value = row['close']
-                                open_value = row['open']
-                                high_value = row['high']
-                                low_value = row['low']
-                                vwap_value = row['vwap']
-
-                                ex_value_value = row['extrinsic_value']
-                                in_value_value = row['intrinsic_value']
-                                iv_percentile_value = row['iv_percentile']
-                                moneyness_value = row['moneyness']
-                                velocity_value = row['velocity']
-                                profit_potential_value = row['opp']
-                                spread_value = row['spread']
-                                spread_pct_value = row['spread_pct']
-                                time_value_value = row['time_value']
-                                trade_size_value = row['trade_size']
-                                trade_price_value = row['price']
-                                trade_exchange_value = row['exchange']
-                                data_dict = { 
-                                    'option_symbol': underlying_symbol_value,
-                                    'strike': strike,
-                                    'call_put': call_put_value,
-                                    'expiry': expiry_value,
-                                    'theta': theta_value,
-                                    'decay': theta_decay_value,
-                                    'bid': bid_value,
-                                    'mid': mid_value,
-                                    'ask': ask_value,
-                                    'vwap': vwap_value,
-                                    'velocity': velocity_value,    
-                                    'intrinsic_value': in_value_value,
-                                    'extrinsic_value': ex_value_value,
-                                    'oi': oi_value,
-                                    'volume': vol_value,
-                                    'iv': iv_value,
-                                    'iv_percentile': iv_percentile_value,  
-                                    'profit_potential': profit_potential_value,
-                                                                    }
-                                if call_put_value == 'call':
-                                    color = hex_color_dict['green']
-                                else:
-                                    color = hex_color_dict['red']
-                              
-
+                                ask_value = row['ask']
+                                bid_value = row['bid']
+                                print(oi_value)
                                 
+
+
+                                if row['cp'] == 'call':
+                                    color = hex_color_dict.get('green')
+                                else:
+                                    color = hex_color_dict.get('red')
+
+                                    
                                 if oi_value >= 100000:
-                                    hook = AsyncDiscordWebhook(os.environ.get('oi_100kplus'))
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('oi_100kplus'), row=row, color=color, description='```py\nThis feed represents options trading with open interest with 100,000 or more.```', status='OI 5k-10k', sector=sector))
+
+
+
+
+                            
+                                if oi_value >= 5000 and oi_value <= 9999:
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('oi_5k10k'), row=row, color=color, description='```py\nThis feed represents options trading with open interest between 5,000 and 10,000```', status='OI 5k-10k', sector=sector))
                                     
 
-                                    embed = DiscordEmbed(title=f"{underlying_symbol_value} | ${underlying_price_value} | {strike_value} | {expiry_value}", description=f"```py\nThese contracts are trading with OI values greater than 100,000 contracts.```", color=color)
-                                    embed.add_embed_field(name=f"Contract Stats:", value=f"> OPEN: **${open_value}**\n> HIGH: **${high_value}**\n> LOW: **${low_value}**\n> CLOSE: **${close_value}**\n> VWAP: **${vwap_value}**\n> CHANGE%: **{change_percent_value}**")
-                                    embed.add_embed_field(name=f"OI / VOl", value=f"> OI: **{oi_value}**\n> VOL: **{vol_value}**\n> RATIO: **{vol_oi_value}**")
-                                    embed.add_embed_field(name=f"IV:", value=f"> **{iv_value}**\n> Percentile: **{iv_percentile_value}**")
-                                    embed.add_embed_field(name=f"Value:", value=f"> Intrinsic: **{in_value_value}**\n> Extrinsic: **{ex_value_value}**\n> Time: **{time_value_value}**")
-                                    embed.add_embed_field(name=f"Spread:", value=f"> Bid: **${bid_value}**\n> Mid: **${mid_value}**\n> Ask: **{ask_value}**\n> Spread: **{round(float(spread_value),2)}**\n> Spread PCT: **{spread_pct_value}%**")
-                                    embed.add_embed_field(name=f"Last Trade", value=f"> Size: **{trade_size_value}**\n> Price: **{trade_price_value}**\n> Exchange: **{trade_exchange_value}**")
-                                    embed.add_embed_field(name="Details:", value=f"> Moneyness: **{moneyness_value}**\n> Velocity: **{velocity_value}**\n> Profit Potential: **{profit_potential_value}**")
-                                    embed.add_embed_field(name=f"GREEKS:", value=f"> Delta: **{delta_value}**\n> Delta/Theta Ratio: **{delta_theta_value}**\n> Gamma: **{gamma_value}**\n> Gamma Risk: **{gamma_risk_value}**\n> Vega: **{vega_value}**\n> Vega Impact: **{vega_impact_value}**\n> Theta: **{theta_value}**\n> Decay Rate: **{theta_decay_value}**", inline=False)
-                                    embed.set_timestamp()
-                                    embed.set_footer(text='OI 100k+ | Data by Polygon.io | Implemented by FUDSTOP')
-                                    asyncio.create_task(self.analyzer.send_hook(hook, embed))
-                                    asyncio.create_task(self.insert_theta_resistant(df, 'oi_100k_plus', unique_columns='option_symbol'))
-                                    asyncio.create_task(asyncio.sleep(1))
-
-                         
-                                if oi_value >= 1000 and oi_value <= 4999:
-                                    df = pd.DataFrame(data_dict)
-                                    hook = AsyncDiscordWebhook(os.environ.get('oi_5k10k'))
-
-                                    embed = DiscordEmbed(title=f"{underlying_symbol_value} | ${underlying_price_value} | {strike_value} | {expiry_value}", description=f"```py\nThese contracts are trading with OI values between 1000 and 5000.```", color=color)
-                                    embed.add_embed_field(name=f"Contract Stats:", value=f"> OPEN: **${open_value}**\n> HIGH: **${high_value}**\n> LOW: **${low_value}**\n> CLOSE: **${close_value}**\n> VWAP: **${vwap_value}**\n> CHANGE%: **{change_percent_value}**")
-                                    embed.add_embed_field(name=f"OI / VOl", value=f"> OI: **{oi_value}**\n> VOL: **{vol_value}**\n> RATIO: **{vol_oi_value}**")
-                                    embed.add_embed_field(name=f"Value:", value=f"> Intrinsic: **{in_value_value}**\n> Extrinsic: **{ex_value_value}**\n> Time: **{time_value_value}**")
-                                    embed.add_embed_field(name=f"Spread:", value=f"> Bid: **${bid_value}**\n> Mid: **${mid_value}**\n> Ask: **{ask_value}**\n> Spread: **{round(float(spread_value),2)}**\n> Spread PCT: **{spread_pct_value}%**")
-                                    embed.add_embed_field(name=f"Last Trade", value=f"> Size: **{trade_size_value}**\n> Price: **{trade_price_value}**\n> Exchange: **{trade_exchange_value}**")
-                                    embed.add_embed_field(name="Details:", value=f"> Moneyness: **{moneyness_value}**\n> Velocity: **{velocity_value}**\n> Profit Potential: **{profit_potential_value}**")
-                                    embed.add_embed_field(name=f"GREEKS:", value=f"> Delta: **{delta_value}**\n> Delta/Theta Ratio: **{delta_theta_value}**\n> Gamma: **{gamma_value}**\n> Gamma Risk: **{gamma_risk_value}**\n> Vega: **{vega_value}**\n> Vega Impact: **{vega_impact_value}**\n> Theta: **{theta_value}**\n> Decay Rate: **{theta_decay_value}**", inline=False)
-
-                                    embed.set_timestamp()
-                                    embed.set_footer(text='OI 1k to 5k | Data by Polygon.io | Implemented by FUDSTOP')
-                                    asyncio.create_task(self.analyzer.send_hook(hook, embed))
-                                    
-                                    asyncio.create_task(asyncio.sleep(1))
-
 
                                 if oi_value >= 10000 and oi_value <= 49999:
-                                    df = pd.DataFrame(data_dict)
-                                    hook = AsyncDiscordWebhook(os.environ.get('oi_10k50k'))
-
-                                    embed = DiscordEmbed(title=f"{underlying_symbol_value} | ${underlying_price_value} | {strike_value} | {expiry_value}", description=f"```py\nThese contracts are trading with OI values between 10,000 and 50,000 contracts.```", color=color)
-                                    embed.add_embed_field(name=f"Contract Stats:", value=f"> OPEN: **${open_value}**\n> HIGH: **${high_value}**\n> LOW: **${low_value}**\n> CLOSE: **${close_value}**\n> VWAP: **${vwap_value}**\n> CHANGE%: **{change_percent_value}**")
-                                    embed.add_embed_field(name=f"OI / VOl", value=f"> OI: **{oi_value}**\n> VOL: **{vol_value}**\n> RATIO: **{vol_oi_value}**")
-                                    embed.add_embed_field(name=f"IV:", value=f"> **{iv_value}**\n> Percentile: **{iv_percentile_value}**")
-                                    embed.add_embed_field(name=f"Value:", value=f"> Intrinsic: **{in_value_value}**\n> Extrinsic: **{ex_value_value}**\n> Time: **{time_value_value}**")
-                                    embed.add_embed_field(name=f"Spread:", value=f"> Bid: **${bid_value}**\n> Mid: **${mid_value}**\n> Ask: **{ask_value}**\n> Spread: **{round(float(spread_value),2)}**\n> Spread PCT: **{spread_pct_value}%**")
-                                    embed.add_embed_field(name=f"Last Trade", value=f"> Size: **{trade_size_value}**\n> Price: **{trade_price_value}**\n> Exchange: **{trade_exchange_value}**")
-                                    embed.add_embed_field(name="Details:", value=f"> Moneyness: **{moneyness_value}**\n> Velocity: **{velocity_value}**\n> Profit Potential: **{profit_potential_value}**")
-                                    embed.add_embed_field(name=f"GREEKS:", value=f"> Delta: **{delta_value}**\n> Delta/Theta Ratio: **{delta_theta_value}**\n> Gamma: **{gamma_value}**\n> Gamma Risk: **{gamma_risk_value}**\n> Vega: **{vega_value}**\n> Vega Impact: **{vega_impact_value}**\n> Theta: **{theta_value}**\n> Decay Rate: **{theta_decay_value}**", inline=False)
-
-                                    embed.set_timestamp()
-                                    embed.set_footer(text='OI 10k to 50k | Data by Polygon.io | Implemented by FUDSTOP')
-                                    asyncio.create_task(self.analyzer.send_hook(hook, embed))
-                                    asyncio.create_task(self.db.batch_insert_dataframe(df, 'oi_10k_50k', unique_columns='option_symbol'))
-                             
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('oi_10k50k'), row=row, color=color, description='```py\nThis feed represents options trading with open interest between 10,000 and 50,000```', status='OI 10k-50k', sector=sector))
+                                
 
 
 
                                 if oi_value >= 50000 and oi_value <= 99999:
-                                    df = pd.DataFrame(data_dict)
-                                    hook = AsyncDiscordWebhook(os.environ.get('oi_50k100k'))
-
-                                    embed = DiscordEmbed(title=f"{underlying_symbol_value} | ${underlying_price_value} | {strike_value} | {expiry_value}", description=f"```py\nThese contracts are trading with OI values between 50,000 and 100,000 contracts.```", color=color)
-                                    embed.add_embed_field(name=f"Contract Stats:", value=f"> OPEN: **${open_value}**\n> HIGH: **${high_value}**\n> LOW: **${low_value}**\n> CLOSE: **${close_value}**\n> VWAP: **${vwap_value}**\n> CHANGE%: **{change_percent_value}**")
-                                    embed.add_embed_field(name=f"OI / VOl", value=f"> OI: **{oi_value}**\n> VOL: **{vol_value}**\n> RATIO: **{vol_oi_value}**")
-                                    embed.add_embed_field(name=f"IV:", value=f"> **{iv_value}**\n> Percentile: **{iv_percentile_value}**")
-                                    embed.add_embed_field(name=f"Value:", value=f"> Intrinsic: **{in_value_value}**\n> Extrinsic: **{ex_value_value}**\n> Time: **{time_value_value}**")
-                                    embed.add_embed_field(name=f"Spread:", value=f"> Bid: **${bid_value}**\n> Mid: **${mid_value}**\n> Ask: **{ask_value}**\n> Spread: **{round(float(spread_value),2)}**\n> Spread PCT: **{spread_pct_value}%**")
-                                    embed.add_embed_field(name=f"Last Trade", value=f"> Size: **{trade_size_value}**\n> Price: **{trade_price_value}**\n> Exchange: **{trade_exchange_value}**")
-                                    embed.add_embed_field(name="Details:", value=f"> Moneyness: **{moneyness_value}**\n> Velocity: **{velocity_value}**\n> Profit Potential: **{profit_potential_value}**")
-                                    embed.add_embed_field(name=f"GREEKS:", value=f"> Delta: **{delta_value}**\n> Delta/Theta Ratio: **{delta_theta_value}**\n> Gamma: **{gamma_value}**\n> Gamma Risk: **{gamma_risk_value}**\n> Vega: **{vega_value}**\n> Vega Impact: **{vega_impact_value}**\n> Theta: **{theta_value}**\n> Decay Rate: **{theta_decay_value}**", inline=False)
-                                    embed.set_timestamp()
-                                    embed.set_footer(text='OI 50k to 100k | Data by Polygon.io | Implemented by FUDSTOP')
-                                    asyncio.create_task(self.analyzer.send_hook(hook, embed))
-           
-                                    asyncio.create_task(self.db.batch_insert_dataframe(df, 'oi_50k_100k', unique_columns='option_symbol'))
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('oi_50k100k'), row=row, color=color, description='```py\nThis feed represents options trading with open interest between 50,000 and 100,000```', status='OI 50k-100k', sector=sector))
+                                    
+            
+                      
 
 
-                                if theta_value <= -0.01 and theta_value >= -0.03 and ask_value >= 0.18 and ask_value <= 1.50:
-                                    df = pd.DataFrame(data_dict)
-                                    hook = AsyncDiscordWebhook(os.environ.get('theta_resistant'))
-                                    embed = DiscordEmbed(title=f"Theta RESISTANT | {underlying_symbol_value} | ${strike_value} | {call_put_value} | {expiry_value}", description=f"```py\nThese contracts have high theta values - meaning they lose value to time decay SLOWER. Good for long-term horizons / hedging.```",color=color)
-                                    embed.add_embed_field(name=f"Contract Stats:", value=f"> OPEN: **${open_value}**\n> HIGH: **${high_value}**\n> LOW: **${low_value}**\n> CLOSE: **${close_value}**\n> VWAP: **${vwap_value}**\n> CHANGE%: **{change_percent_value}**")
-                                    embed.add_embed_field(name=f"OI / VOl", value=f"> OI: **{oi_value}**\n> VOL: **{vol_value}**\n> RATIO: **{vol_oi_value}**")
-                                    embed.add_embed_field(name=f"IV:", value=f"> **{iv_value}**\n> Percentile: **{iv_percentile_value}**")
-                                    embed.add_embed_field(name=f"Value:", value=f"> Intrinsic: **{in_value_value}**\n> Extrinsic: **{ex_value_value}**\n> Time: **{time_value_value}**")
-                                    embed.add_embed_field(name=f"Spread:", value=f"> Bid: **${bid_value}**\n> Mid: **${mid_value}**\n> Ask: **{ask_value}**\n> Spread: **{round(float(spread_value),2)}**\n> Spread PCT: **{spread_pct_value}%**")
-                                    embed.add_embed_field(name=f"Last Trade", value=f"> Size: **{trade_size_value}**\n> Price: **{trade_price_value}**\n> Exchange: **{trade_exchange_value}**")
-                                    embed.add_embed_field(name="Details:", value=f"> Moneyness: **{moneyness_value}**\n> Velocity: **{velocity_value}**\n> Profit Potential: **{profit_potential_value}**")
-                                    embed.add_embed_field(name=f"GREEKS:", value=f"> Delta: **{delta_value}**\n> Delta/Theta Ratio: **{delta_theta_value}**\n> Gamma: **{gamma_value}**\n> Gamma Risk: **{gamma_risk_value}**\n> Vega: **{vega_value}**\n> Vega Impact: **{vega_impact_value}**\n> Theta: **{theta_value}**\n> Decay Rate: **{theta_decay_value}**", inline=False)
-                                    embed.set_timestamp()
-                                    embed.set_footer(text='Theta Resistant | Data by Polygon.io | Implemented by FUDSTOP')
+                                if row['theta'] is not None and row['theta'] <= -0.01 and row['theta'] >= -0.03 and ask_value >= 0.18 and ask_value <= 1.50:
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('theta_resistant'), row=row, color=color, description='```py\nThese feeds represent THETA RESISTANT options, or option trades that have theta values between -0.01 and -0.03, or lose between $1 and $3 per day.```', status='Theta Resistant', sector=sector))
 
 
 
+                                if row['ticker'] in ['SPY', 'IWM', 'QQQ', 'TQQQ', 'SQQQ'] and row['vol'] >= 5000:
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('index_surge'), row=row, color=color, description='```py\nThese feeds represent INDEX SURGES, or option trades with 5k+ volume that occur on index tickers such as SPY, QQQ, IWM, etc.```', status='Index Surge', sector=sector))
 
-                                    asyncio.create_task(self.analyzer.send_hook(hook, embed))
-                                    asyncio.create_task(asyncio.sleep(1))
-                                    asyncio.create_task(self.insert_theta_resistant(df, 'theta_resistant', unique_columns='option_symbol'))
+
+                                if row['iv_percentile'] is not None and row['iv_percentile'] <= 0.25 and row['theta'] is not None and row['theta'] <= -0.01 and row['theta'] >= -0.06:
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('iv_percentile'), row=row, color=color, description='```py\nThese feeds represent LOW IV PERCENTILE, meaning IV is low relative to the historical measurement over the last year. Also - Theta values will range between -0.01 and -0.06 in these feeds.```', status='IV Percentile', sector=sector))
+
+
+                                if row['vol'] is not None and row['vol'] >= 500 and row['vol'] <= 999:
+                                    sector = await identify_sector(row['ticker'])
+                                    
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('vol_5001k'), row=row, color=color, description='```py\nThis feed represents an option trade with volume between 500 and 1,000 contracts.```', status='vol 500-1k', sector=sector))
+                                if row['vol'] is not None and row['vol'] >= 1000 and row['vol'] <= 9999:
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('vol_1k10k'), row=row, color=color, description='```py\nThis feed represents an option trade with volume between 1,000 and 10,000 contracts.```', status='vol 1k-10k', sector=sector))
+                                if row['vol'] is not None and row['vol'] >= 10000 and row['vol'] <= 49999:
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('vol_10k50k'), row=row, color=color, description='```py\nThis feed represents an option trade with volume between 10,000 and 50,000 contracts.```', status='vol 10k-50k', sector=sector))
+                                if row['vol'] is not None and row['vol'] >= 50000:
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('vol_50kplus'), row=row, color=color, description='```py\nThis feed represents an option trade with volume of more than 50,000 contracts.```', status='vol 50k+', sector=sector))
+
+
+                                if row['change_percent'] <= -65 and row['dte'] > 1:
+                                    sector = await identify_sector(row['ticker'])
+                                    asyncio.create_task(conditional_embed(webhook=os.environ.get('opt_change_percent'), row=row, color=hex_color_dict.get('red'), description='```py\nThis feed represents options that are down 65% or less on the day.```', status='Change% -65% or less', sector=sector))
+
+
+
+                        except Exception as e:
+                            print(e)
+                            continue
 
 
 
@@ -489,20 +420,18 @@ class MultiMarkets(Inserts):
 
 
             #stock aggs
-            elif event_type == 'A' and not m.symbol.startswith('I:') and not m.symbol.startswith("O:"):
+            if event_type == 'A' and not m.symbol.startswith('I:') and not m.symbol.startswith("O:"):
                 if m.symbol in set(most_active_tickers):
                     async for data in self.db.insert_stock_aggs(m):
                         ticker = data.get('ticker')
                         
                         await self.queue.put(data)
 
-                        asyncio.create_task(self.stock_rsi(ticker))
-                        asyncio.create_task(self.stock_macd(ticker))
 
 
             #option trades
             
-            elif event_type == 'T' and m.symbol.startswith('O:'):
+            if event_type == 'T' and m.symbol.startswith('O:'):
                 
                 async for data in self.db.insert_option_trades(m):
                     size = data.get('size')
@@ -523,28 +452,21 @@ class MultiMarkets(Inserts):
                     exchange = data.get('exchange')
                     self.trade_tickers.append(ticker)
                 
-
+                    hook = option_conditions_hooks[data.get('conditions')]
                     if data.get('size') is not None and data.get('size') >= 500 and data.get('conditions') in option_conditions_hooks:
-                        hook = option_conditions_hooks[data.get('conditions')]
+                        
                         asyncio.create_task(option_condition_embed(price_to_strike=data.get('price_to_strike'),conditions=data.get('conditions'), option_symbol=ticker,underlying_symbol=symbol,strike=data.get('strike'),call_put=data.get('call_put'),expiry=data.get('expiry'),price = data.get('price'), size=data.get('size'),exchange = data.get('exchange'),volume_change = data.get('volume_change'),price_change=data.get('price_change'),weekday=data.get('weekday'),hour_of_day=data.get('hour_of_day'), hook=hook))
 
 
     
 
 
-                    if size is not None and size >= 10000 and size <= 49999:
-                        asyncio.create_task(sized_trade_embed(dollar_cost=dollar_cost,expiry=expiry,option_symbol=ticker,call_put=call_put,strike=strike,underlying_symbol=symbol,price=price,price_change=price_change,size=size,volume_change=volume_change,conditions=conditions,exchange=exchange,price_to_strike=data.get('price_to_strike'),hour_of_day=hour_of_day,weekday=weekday))
-
-                    if size is not None and size >= 50000:
-                        asyncio.create_task(sized_trade_embed(dollar_cost=dollar_cost,expiry=expiry,option_symbol=ticker,call_put=call_put,strike=strike,underlying_symbol=symbol,price=price,price_change=price_change,size=size,volume_change=volume_change,conditions=conditions,exchange=exchange,price_to_strike=data.get('price_to_strike'),hour_of_day=hour_of_day,weekday=weekday))
 
 
 
             #stock trades
-            elif event_type == 'T' and not m.symbol.startswith('O:') and not m.symbol.startswith('I:'):
+            if event_type == 'T' and not m.symbol.startswith('O:') and not m.symbol.startswith('I:'):
                 async for data in self.db.insert_stock_trades(m):
-                
-                    await self.queue.put(data)
 
                     ticker = data.get('ticker')
                     if ticker in set(most_active_tickers):
@@ -605,7 +527,7 @@ class MultiMarkets(Inserts):
             elif event_type == 'XT':
                 async for data in self.db.insert_crypto_trades(m):
                     ticker = data.get('ticker')
-                    print(f"CRYPTO TRADES: {ticker}")
+
                     # Call the repeated_hits method
                     last_five_trades = await monitor.repeated_hits(data)
 
@@ -624,7 +546,19 @@ class MultiMarkets(Inserts):
             elif event_type == 'CAS':
                 async for data in self.db.insert_forex_aggs(m=m):
                     ticker = data.get('ticker')
-                    print(f"FOREX AGGS: {ticker}")
+
+                    if ticker in self.db.currency_pairs and data.get('volume') >= 2:
+                        random_color_name = random.choice(list(hex_color_dict.values()))
+                      
+                        hook = AsyncDiscordWebhook(self.db.forex_hook_dict.get(ticker))
+                        embed = DiscordEmbed(title=f"Forex Trades - {data.get('ticker')} || {data.get('name')}", description=f"```py\nThis feed represents live forex trades for the pair {data.get('name')}\n{self.db.pair_to_description.get(data.get('ticker'))}```", color=random_color_name)
+                        embed.add_embed_field(name=f"Stats:", value=f"> Open: **${data.get('open')}**\n> High: **${data.get('high')}**\n> Low: **${data.get('low')}**\n> Now: **${data.get('close')}**\n>")
+                        embed.add_embed_field(name=f"Volume:", value=f"> **{data.get('volume')}**")
+                        embed.set_timestamp()
+                        embed.set_footer(text='Data by Polygon.io | Implemented by FUDSTOP', icon_url=os.environ.get('fudstop_logo'))
+                        hook.add_embed(embed)
+
+                        await hook.execute()
 
 
     async def insert_new_prices(self, ticker, type, fifty_high, price, fifty_low, timestamp):
@@ -650,7 +584,7 @@ market = MultiMarkets(host='localhost', user='chuck', database='markets', port=5
 
 
 async def main():
-    await market.db.connect()
+    
     while True:  # Restart mechanism
         try:
             await run_main_tasks()
@@ -662,7 +596,7 @@ async def main():
 
 # Main async function to connect to all markets with their respective subscriptions
 async def run_main_tasks():
-
+    await market.db.connect()
     clients = []
     for live_market in market.markets:
         patterns = market.subscription_patterns.get(live_market, [])
