@@ -18,7 +18,7 @@ async def main(ticker):
     
     all_options = await db.get_option_chain_all(underlying_asset=ticker)
 
-    asyncio.create_task(db.batch_insert_dataframe(all_options.df,table_name='poly_opts', unique_columns='option_symbol'))
+    asyncio.create_task(db.batch_insert_dataframe(all_options.df,table_name='opts', unique_columns='option_symbol'))
 async def run_main():
     await db.connect()
     tasks = [main(i) for i in most_active_tickers]
