@@ -2,15 +2,15 @@
 import sys
 from pathlib import Path
 # Add the project directory to the sys.path
-project_dir = str(Path(__file__).resolve().parents[1])
+project_dir = str(Path(__file__).resolve().parents[2])
 if project_dir not in sys.path:
     sys.path.append(project_dir)
 import os
 from dotenv import load_dotenv
 load_dotenv()
 from discord_webhook import AsyncDiscordWebhook, DiscordEmbed
-from webhook_dicts import option_conditions_hooks
-from embeddings import sized_trade_embed, specials_embed_2, dip_specials_embed, index_surge_embed, dip_specials_embed_2_super, theta_resistant_embed
+from _markets.webhook_dicts import option_conditions_hooks
+from _markets.embeddings import sized_trade_embed, specials_embed_2, dip_specials_embed, index_surge_embed, dip_specials_embed_2_super, theta_resistant_embed
 from polygon.websocket import WebSocketClient, WebSocketMessage, Market, Feed
 from apis.polygonio.polygon_options import PolygonOptions
 from webull_options.webull_options import WebullOptions
@@ -39,7 +39,7 @@ class OptionsMarket:
         self.webull = WebullTrading()
         self.polygon = Polygon(database=database,user=user,password=password,port=port,host=host)
         self.poly_opts = PolygonOptions(user=user, database=database, host=host, port=port, password=password)
-        self.wb_opts = WebullOptions(user='chuck', database='charlie', host='localhost', port=5432, password='fud')
+        self.wb_opts = WebullOptions(user='chuck', database='markets')
         self.conn = None
         self.pool = None
         self.test = os.environ.get('test')
