@@ -12,7 +12,7 @@ YOUR_API_KEY = os.environ.get("YOUR_POLYGON_KEY")
 
 import re
 import pandas as pd
-from apis.polygonio.mapping import OPTIONS_EXCHANGES,option_condition_dict,STOCK_EXCHANGES,stock_condition_dict,TAPES
+
 
 from colorsys import rgb_to_hsv
 from bs4 import BeautifulSoup
@@ -56,20 +56,26 @@ def flatten_object(obj, parent_key='', separator='_'):
 
 def format_large_number(number):
     """
-    Formats a number into a human-readable format (e.g., 1K, 1M, 1B, etc.)
+    Formats a number into a human-readable format (e.g., 1K, 1M, 1B, etc.) including negative numbers.
+    Handles NoneType to avoid TypeError.
     """
-    if pd.isnull(number):
-        return number  # Return NaN as is
-    if number < 1000:
-        return str(number)
-    elif 1000 <= number < 1000000:
-        return f"{number/1000:.1f}K"
-    elif 1000000 <= number < 1000000000:
-        return f"{number/1000000:.1f}M"
-    elif 1000000000 <= number:
-        return f"{number/1000000000:.1f}B"
+    if number is None:
+        return 0.0  # or any placeholder you prefer for None values
+    prefix = "-" if number < 0 else ""
+    abs_number = abs(number)
+    
+    if abs_number < 1000:
+        return f"{prefix}{abs_number}"
+    elif abs_number < 1000000:
+        return f"{prefix}{abs_number/1000:.1f}K"
+    elif abs_number < 1000000000:
+        return f"{prefix}{abs_number/1000000:.1f}M"
     else:
-        return str(number)
+        return f"{prefix}{abs_number/1000000000:.1f}B"
+def lowercase_columns(df):
+    # Rename columns to lowercase after converting them to strings
+    df.columns = map(lambda x: str(x).lower(), df.columns)
+    return df
 
 def format_large_numbers_in_dataframe(df):
     """
@@ -82,7 +88,10 @@ def format_large_numbers_in_dataframe(df):
         formatted_df[column] = formatted_df[column].apply(format_large_number)
     
     return formatted_df
-
+def chunk_data(data, chunk_size):
+    """Yield successive chunk_size chunks from data."""
+    for i in range(0, len(data), chunk_size):
+        yield data[i:i + chunk_size]
 # Example function to format selected keys in a dictionary
 def format_large_numbers_in_dict(data_dict, keys_to_format):
     """
@@ -336,16 +345,7 @@ def calculate_price_to_strike(price, strike):
     return price / strike if strike != 0 else 0
 
 
-def map_conditions(conditions):
-    if isinstance(conditions, list):
-        return [option_condition_dict.get(cond, cond) for cond in conditions]
-    else:
-        return conditions
-def map_stock_conditions(conditions):
-    if isinstance(conditions, list):
-        return [stock_condition_dict.get(cond, cond) for cond in conditions]
-    else:
-        return conditions
+
 def count_itm_otm(group):
     underlying_price = group['underlying_price'].iloc[0]  # Assuming the underlying price is the same within each expiry group
     itm_call = len(group[(group['call_put'] == 'call') & (group['strike'] < underlying_price)])
@@ -589,106 +589,6 @@ async def fetch_url(session, url):
 
 
 
-
-async def process_contract(data, db):
-        
-
-
-    if data is not None:
-        for contract in data:
-            try:
-
-                option_symbol = contract.get('ticker')
-                expiry = contract.get('details').get('expiration_date')
-                strike = contract.get('details').get('strike_price')
-                call_put = contract.get('details').get('contract_type')
-                underlying_symbol = contract.get('underlying_asset').get('ticker')
-                underlying_price = contract.get('underlying_asset').get('price')
-                change_to_breakeven = contract.get('underlying_asset').get('change_to_breakeven')
-
-                
-
-                trade_size = contract.get('last_trade').get('size')
-                trade_price = contract.get('last_trade').get('price')
-                trade_conditions = contract.get('last_trade', None).get('conditions', None)
-                sip_timestamp = contract.get('last_trade', None).get('sip_timestamp', None)
-                if sip_timestamp is not None:
-                    sip_timestamp = datetime.fromtimestamp(sip_timestamp / 1e9)
-                trade_conditions = [option_condition_dict.get(c) for c in trade_conditions] if trade_conditions is not None else []
-                trade_conditions = trade_conditions[0]
-                trade_exchange = OPTIONS_EXCHANGES.get(contract.get('last_trade').get('exchange', None))
-                break_even_price = contract.get('break_even_price', None)
-                implied_volatility = contract.get('implied_volatility', None)
-                open_interest = contract.get('open_interest', None)
-                name = contract.get('name', None)
-                open = contract.get('session').get('open', None)
-                high = contract.get('session').get('high', None)
-                low = contract.get('session').get('low', None)
-                close = contract.get('session').get('close', None)
-                volume = contract.get('session').get('volume', None)
-                prev_close = contract.get('session').get('previous_close')
-                early_trading_change_percent = contract.get('session').get('early_trading_change_percent')
-                change = contract.get('session').get('change')
-                early_trading_change = contract.get('session').get('early_trading_change')
-                change_percent = contract.get('session').get('change_percent', None)
-
-                delta = contract.get('greeks').get('delta')
-                vega = contract.get('greeks').get('vega')
-                theta = contract.get('greeks').get('theta')
-                gamma = contract.get('greeks').get('gamma')
-
-                ask = contract.get('last_quote').get('ask')
-                bid = contract.get('last_quote').get('bid')
-                ask_size = contract.get('last_quote').get('ask_size')
-                bid_size = contract.get('last_quote').get('bid_size')
-                ask_exchange = OPTIONS_EXCHANGES.get(contract.get('last_quote').get('ask_exchange'))
-                bid_exchange = OPTIONS_EXCHANGES.get(contract.get('last_quote').get('bid_exchange'))
-
-
-                data_dict = {
-                    'option_symbol': option_symbol,
-                    'expiry': expiry,
-                    'strike': strike,
-                    'call_put': call_put,
-                    'underlying_symbol': underlying_symbol,
-                    'underlying_price': underlying_price,
-                    'change_to_breakeven': change_to_breakeven,
-                    'trade_size': trade_size,
-                    'trade_price': trade_price,
-                    'trade_conditions': trade_conditions,
-                    'sip_timestamp': sip_timestamp,
-                    'trade_exchange': trade_exchange,
-                    'break_even_price': break_even_price,
-                    'implied_volatility': implied_volatility,
-                    'open_interest': open_interest,
-                    'name': name,
-                    'open': open,
-                    'high': high,
-                    'low': low,
-                    'close': close,
-                    'volume': volume,
-                    'prev_close': prev_close,
-                    'early_trading_change_percent': early_trading_change_percent,
-                    'change': change,
-                    'early_trading_change': early_trading_change,
-                    'change_percent': change_percent,
-                    'delta': delta,
-                    'vega': vega,
-                    'theta': theta,
-                    'gamma': gamma,
-                    'ask': ask,
-                    'bid': bid,
-                    'ask_size': ask_size,
-                    'bid_size': bid_size,
-                    'ask_exchange': ask_exchange,
-                    'bid_exchange': bid_exchange
-                }
-
-
-                return data_dict
-            except Exception as e:
-                # Handle any other exceptions that might occur
-                print(f"An error occurred: {e}")
 from asyncio import Semaphore
 sema = Semaphore(5)
 
@@ -1139,3 +1039,37 @@ def save_image(filename: str, fig: go.Figure = None, bytesIO: io.BytesIO = None)
     image.close()
 
     return imagefile
+
+
+
+from fudstop._markets.list_sets.dicts import healthcare,energy,industrials,utilities,etfs,technology,consumer_cyclical,consumer_defensive,communication_services,financial_services,real_estate,basic_materials
+async def identify_sector(ticker):
+    if ticker in healthcare:
+        sector = 'Healthcare'
+    elif ticker in energy:
+        sector = 'Energy'
+    elif ticker in industrials:
+        sector = 'Industrials'
+    elif ticker in utilities:
+        sector = 'Utilities'
+    elif ticker in etfs:
+        sector = 'ETFs'
+    elif ticker in technology:
+        sector = 'Technology'
+    elif ticker in consumer_cyclical:
+        sector = 'ConsumerCyclical'
+    elif ticker in consumer_defensive:
+        sector = 'ConsumerDefensive'
+    elif ticker in communication_services:
+        sector = 'CommunicationServices'
+    elif ticker in financial_services:
+        sector = 'FinancialServices'
+    elif ticker in real_estate:
+        sector = 'RealEstate'
+    elif ticker in basic_materials:
+        sector = 'BasicMaterials'
+    else:
+        sector = 'Unknown'
+
+
+    return sector
\ No newline at end of file
