@@ -2,24 +2,26 @@ import sys
 from pathlib import Path
 from asyncpg.exceptions import UniqueViolationError
 # Add the project directory to the sys.path
-project_dir = str(Path(__file__).resolve().parents[2])
+project_dir = str(Path(__file__).resolve().parents[1])
 if project_dir not in sys.path:
     sys.path.append(project_dir)
-
+import httpx
 from dotenv import load_dotenv
 load_dotenv()
 from asyncpg import create_pool
 from urllib.parse import unquote
 import os
+from fudstop.apis.helpers import format_large_numbers_in_dataframe
 from typing import List, Dict, Optional
 import pandas as pd
 import asyncio
 from aiohttp.client_exceptions import ClientConnectorError, ClientOSError, ClientConnectionError, ContentTypeError
-
-from .models.aggregates import AggregatesData
+from fudstop.apis.webull.webull_trading import WebullTrading
+from .models.aggregates import Aggregates
 from .models.ticker_news import TickerNews
 from .models.company_info import CombinedCompanyResults
 from .models.technicals import RSI, EMA, SMA, MACD
+from .models.gainers_losers import GainersLosers
 from .models.ticker_snapshot import StockSnapshot
 from .models.trades import TradeData, LastTradeData
 from datetime import datetime, timedelta
@@ -27,7 +29,7 @@ import aiohttp
 
 from urllib.parse import urlencode
 import requests
-from apis.helpers import flatten_dict
+from fudstop.apis.helpers import flatten_dict
 
 YOUR_POLYGON_KEY = os.environ.get('YOUR_POLYGON_KEY')
 
@@ -50,6 +52,7 @@ class Polygon:
         self.fifteen_days_from_now = (datetime.now() + timedelta(days=15)).strftime('%Y-%m-%d')
         self.eight_days_from_now = (datetime.now() + timedelta(days=8)).strftime('%Y-%m-%d')
         self.eight_days_ago = (datetime.now() - timedelta(days=8)).strftime('%Y-%m-%d')
+        self.trading = WebullTrading()
         self.timeframes = ['minute', 'hour','day', 'week', 'month']
         self.session = None
     async def create_session(self):
@@ -138,9 +141,9 @@ class Polygon:
                         break
         if as_dataframe:
             import pandas as pd
-            yield pd.DataFrame(all_results)
+            return pd.DataFrame(all_results)
         else:
-            yield all_results
+            return all_results
         
 
     async def fetch_endpoint(self, url):
@@ -174,7 +177,7 @@ class Polygon:
         await self.session.close()
                     
 
-    async def aggregates(self, ticker, multiplier:str='1', timespan:str='day', date_from=None, date_to=None, limit:str=500, sort:str='desc'):
+    async def get_aggs(self, ticker:str='AAPL', multiplier:int=1, timespan:str='second', date_from:str='2024-01-01', date_to:str='2024-04-12', adjusted:str='true', sort:str='asc', limit:int=50000):
         """
         Fetches candlestick data for a ticker, option symbol, crypto/forex pair.
         
@@ -214,31 +217,23 @@ class Polygon:
         >>> await aggregates('AAPL', date_from='2023-09-01', date_to='2023-10-01')
         """
 
-        if date_from is None:
-            date_from = (datetime.now() - timedelta(days=30)).strftime('%Y-%m-%d')
-        
-        if date_to is None:
-            date_to = datetime.now().strftime('%Y-%m-%d')
-        
-  
-        params = {
-            'adjusted': 'true',
-            'sort': sort,
-            'limit': limit,
-            'apiKey': self.api_key  # API key included here
-        }
 
-        endpoint = f"https://api.polygon.io/v2/aggs/ticker/{ticker}/range/{multiplier}/{timespan}/{date_from}/{date_to}"
-        if limit == 50000:
-            all_data = await self.paginate_concurrent(endpoint)
-            return all_data    
-        data = await self.fetch_endpoint(endpoint, params=params)
-        data = AggregatesData(data)
+        endpoint = f"https://api.polygon.io/v2/aggs/ticker/{ticker}/range/{multiplier}/{timespan}/{date_from}/{date_to}?adjusted={adjusted}&sort={sort}&limit={limit}&apiKey={os.environ.get('YOUR_POLYGON_KEY')}"
+
+        async with httpx.AsyncClient() as client:
+            data = await client.get(endpoint)
+
+            data = data.json()
+
+            results = data['results'] if 'results' in data else None
+
+            if results is not None:
+
+                results = Aggregates(results, ticker)
+
+
+                return results
 
-        if limit == 50000:
-            all_data = await self.paginate_concurrent(endpoint)
-            return all_data    
-        return data
 
 
     async def market_news(self, limit: str = '100'):
@@ -263,6 +258,36 @@ class Polygon:
         return data
     
 
+    async def dark_pools(self, ticker:str, multiplier:int, timespan:str, date_from:str, date_to:str):
+
+        aggs = await self.get_aggs(ticker=ticker, multiplier=multiplier, timespan=timespan, date_from=date_from, date_to=date_to)
+
+
+
+        # Assuming 'aggs' is an instance of the Aggregates class with populated data
+        dollar_cost_above_10m_details = [
+            {'Close Price': aggs.close[i], 'Timestamp': aggs.timestamp[i], 'Dollar Cost': cost}
+            for i, cost in enumerate(aggs.dollar_cost) 
+            if cost > 10000000
+        ]
+
+        # Create DataFrame from the list of dictionaries
+        df_dollar_cost_above_10m = pd.DataFrame(dollar_cost_above_10m_details)
+
+        # Print the DataFrame to see the result
+        df = format_large_numbers_in_dataframe(df_dollar_cost_above_10m)
+
+        return df
+
+    async def top_gainers_losers(self, type:str='gainers'):
+        endpoint = f"https://api.polygon.io/v2/snapshot/locale/us/markets/stocks/{type}?apiKey={self.api_key}"
+
+        async with httpx.AsyncClient() as client:
+            data = await client.get(endpoint)
+            data = data.json()
+            tickers = data['tickers'] if 'tickers' in data else None
+            return GainersLosers(tickers)
+
     async def company_info(self, ticker) -> CombinedCompanyResults:
         url = f"https://api.polygon.io/v3/reference/tickers/{ticker}?apiKey={self.api_key}"
         await self.create_session()
@@ -392,7 +417,7 @@ class Polygon:
         finally:
             await self.close_session()
 
-    async def rsi(self, ticker:str, timespan:str, limit:str='1000', window:int=14, date_from:str=None, date_to:str=None, session=None, snapshot:bool=False):
+    async def rsi(self, ticker:str, timespan:str, limit:str='1', window:int=14, date_from:str=None, date_to:str=None, session=None, snapshot:bool=False):
         """
         Arguments:
 
@@ -416,26 +441,31 @@ class Polygon:
         >>> *SNAPSHOT: scan all timeframes for a ticker
 
         """
-
-        if date_from is None:
-            date_from = self.eight_days_ago
-
-        if date_to is None:
-            date_to = self.today
-
-
-        endpoint = f"https://api.polygon.io/v1/indicators/rsi/{ticker}?timespan={timespan}&timestamp.gte={date_from}&timestamp.lte={date_to}&limit={limit}&window={window}&apiKey={self.api_key}"
-
         try:
-            async with self.session.get(endpoint) as resp:
-                datas = await resp.json()
-                if datas is not None:
+            if date_from is None:
+                date_from = self.eight_days_ago
 
-                        
+            if date_to is None:
+                date_to = self.today
 
+
+            endpoint = f"https://api.polygon.io/v1/indicators/rsi/{ticker}?timespan={timespan}&timestamp.gte={date_from}&timestamp.lte={date_to}&limit={limit}&window={window}&apiKey={self.api_key}"
+    
+
+            async with aiohttp.ClientSession() as session:
+                try:
+                    async with session.get(endpoint) as response:
+                        datas = await response.json()
+                        print(datas)
+                        if datas is not None:
+
+                            
+
+                    
+                            return RSI(datas, ticker)
+                except ClientOSError as e:
+                    print(f'ERROR {e}')
                 
-                    return RSI(datas, ticker)
-            
 
             if snapshot == True:
                 tasks = []
@@ -447,6 +477,7 @@ class Polygon:
 
 
 
+
     async def macd(self, ticker:str, timespan:str, limit:str='1000'):
         """
         Arguments:
@@ -470,6 +501,7 @@ class Polygon:
 
         
         endpoint = f"https://api.polygon.io/v1/indicators/macd/{ticker}?timespan={timespan}&adjusted=true&short_window=12&long_window=26&signal_window=9&series_type=close&order=desc&apiKey={self.api_key}&limit={limit}"
+        await self.create_session()
         try:
 
             try:
@@ -518,6 +550,7 @@ class Polygon:
 
 
         endpoint = f"https://api.polygon.io/v1/indicators/sma/{ticker}?timespan={timespan}&window={window}&timestamp.gte={date_from}&timestamp.lte={date_to}&limit={limit}&apiKey={self.api_key}"
+        await self.create_session()
         try:
             
 
@@ -526,8 +559,8 @@ class Polygon:
 
 
                 return SMA(datas, ticker)
-        except Exception as e:
-            print(e)
+        finally:
+            await self.close_session()
 
 
     async def ema(self, ticker:str, timespan:str, limit:str='1', window:str='21', date_from:str=None, date_to:str=None):
@@ -564,6 +597,7 @@ class Polygon:
 
         
         try:
+            await self.create_session()  # Ensure the session is created
             async with self.session.get(endpoint) as resp:
                 datas = await resp.json()
                 return EMA(datas, ticker)
@@ -605,13 +639,58 @@ class Polygon:
                 except Exception as e:
                     print(f"An unexpected error occurred: {e}")  # Consider logging this
                     continue
+            except Exception as e:
+                print(e)
+    async def get_price(self, ticker:str):
+        """
+        Fetches price from Webull API to use for option queries
 
-            finally:
-                await self.close_session()
+        Arguments:
+
+        >>> ticker: required - the ticker to survey
+        """
+
+        if ticker.startswith('I:'):
+            ticker = ticker.replace('I:', '')
+        datas = await self.trading.stock_quote(ticker)
+
+        price = datas.web_stock_close
+        if price is not None:
+            return float(price[0])
+
+    async def rsi_snapshot(self, ticker):
+
+        price = await self.get_price(ticker)
+        if price is not None:
+            rsis = {}
+            rsis.update({'ticker': ticker,
+                        'price': price})
+            drsi = asyncio.create_task(self.rsi(ticker, timespan='day'))
+            hrsi = asyncio.create_task(self.rsi(ticker, timespan='hour'))
+            wrsi = asyncio.create_task(self.rsi(ticker, timespan='week'))
+            mrsi = asyncio.create_task(self.rsi(ticker, timespan='month'))
+            day_rsi, hour_rsi, week_rsi, month_rsi = await asyncio.gather(drsi, hrsi, wrsi, mrsi)
+            if day_rsi is not None and hasattr(day_rsi, 'rsi_value') and day_rsi.rsi_value is not None and len(day_rsi.rsi_value) > 0:
+                day_rsi = day_rsi.rsi_value[0]
+
+                rsis.update({'day_rsi': day_rsi})
+
+            if hour_rsi is not None and hasattr(hour_rsi, 'rsi_value') and hour_rsi.rsi_value is not None and len(hour_rsi.rsi_value) > 0:
+                hour_rsi = hour_rsi.rsi_value[0]
+                rsis.update({'hour_rsi': hour_rsi})
 
 
+            if week_rsi is not None and hasattr(week_rsi, 'rsi_value') and week_rsi.rsi_value is not None and len(week_rsi.rsi_value) > 0:
+                week_rsi = week_rsi.rsi_value[0]
+                rsis.update({'week_rsi': week_rsi})
+            if month_rsi is not None and hasattr(month_rsi, 'rsi_value') and month_rsi.rsi_value is not None and len(month_rsi.rsi_value) > 0:
+                month_rsi = month_rsi.rsi_value[0]
+                rsis.update({'month_rsi': month_rsi})
 
+            
+            df = pd.DataFrame(rsis, index=[0])
 
+            return df
 
     async def gather_rsi_for_all_tickers(self, tickers) -> List[RSI]:
 
