@@ -217,7 +217,7 @@ class UniversalOptionSnapshot:
 
 
 
-        details = [i['details'] for i in results]
+        details = [i['details'] if 'details' in i else 0 for i in results]
         self.strike = [float(i['strike_price']) if 'strike_price' in i else None for i in details]
         self.expiry = [i['expiration_date'] if 'expiration_date' in i else None for i in details]
         # Convert the expiration dates into a pandas Series
@@ -319,7 +319,8 @@ class UniversalOptionSnapshot:
         self.opp = [round(item, 3) if item is not None else None for item in self.opp]
 
 
-
+        iv_series = pd.Series(self.implied_volatility).dropna()
+        self.iv_percentile = [round(x, 2) for x in iv_series.rank(pct=True)]
                 
 
 
@@ -335,8 +336,6 @@ class UniversalOptionSnapshot:
 
 
 
-
-
 
 
         self.data_dict = {
@@ -818,7 +817,7 @@ class OptionData:
         self.change_percent = [round(float(session_data.get('change_percent', 0)),2) for session_data in session]
         self.early_change_percent = [round(float(session_data.get('early_trading_change_percent', 0)),2) for session_data in session]
         self.late_change_percent = [round(float(session_data.get('late_trading_change_percent', 0)),2) for session_data in session]
-        details = [i['details'] for i in results]
+        details = [i.get('details', {}) for i in results]
         self.strike = [float(i['strike_price']) if 'strike_price' in i else None for i in details]
         self.expiry = [i['expiration_date'] if 'expiration_date' in i else None for i in details]
         # Convert the expiration dates into a pandas Series
@@ -829,7 +828,7 @@ class OptionData:
         self.exercise_style = [i['exercise_style'] if 'exercise_style' in i else None for i in details]
         self.ticker = [i['ticker'] if 'ticker' in i else None for i in results]
 
-        greeks = [i.get('greeks') for i in results]
+        greeks = [i.get('greeks', {}) for i in results]
         self.theta = [round(float(i.get('theta', 0)), 4) for i in greeks if i is not None and i.get('theta') is not None]
         self.delta = [round(float(i.get('delta', 0)), 4) for i in greeks if i is not None and i.get('delta') is not None]
         self.gamma = [round(float(i.get('gamma', 0)), 4) for i in greeks if i is not None and i.get('gamma') is not None]
@@ -926,11 +925,11 @@ class OptionData:
 
 
 
-        # Weighted Implied Volatility
+
         total_oi = sum(filter(None, self.open_interest))  # Sum of all non-None open interest values
         self.weighted_iv = round(sum(iv * oi for iv, oi in zip(self.implied_volatility, self.open_interest) if iv is not None and oi is not None) / total_oi, 6)
 
-        # Theta Decay Rate (average)
+  
         self.avg_theta_decay = round(sum(filter(None, self.theta)) / len(self.theta), 2)
 
         # IV Percentile
@@ -1008,13 +1007,26 @@ class OptionData:
         }
 
 
-        self.pad_data()
-        try:
-            self.as_dataframe = pd.DataFrame(self.data_dict)
-        except Exception as e:
-            print(e)
+        # Check and correct the data lengths
+        for key, value in self.data_dict.items():
+            if len(value) < 25:
+                # Fill in missing data with None for keys with length 24
+                self.data_dict[key] += [None] * (25 - len(value))
 
-    
+        # Create the DataFrame
+        self.as_dataframe = self.ensure_uniform_length(self.data_dict)
+    def ensure_uniform_length(self, data_dict):
+        # Determine the maximum length among all lists in the dictionary
+        max_length = max(len(value) for value in data_dict.values())
+
+        # Pad shorter lists with None to match the maximum length
+        for key, value in data_dict.items():
+            additional_length = max_length - len(value)
+            if additional_length > 0:
+                data_dict[key] = value + [None] * additional_length
+
+        # Now, all lists in data_dict have the same length and can be safely converted to a DataFrame
+        return pd.DataFrame(data_dict)
     def __getitem__(self, index):
         return self.df[index]
 
