@@ -8,6 +8,7 @@ if project_dir not in sys.path:
     sys.path.append(project_dir)
 from tabulate import tabulate
 from dotenv import load_dotenv
+import httpx
 load_dotenv()
 from .models.technicals import RSI
 from aiohttp.client_exceptions import ClientConnectionError, ClientConnectorError, ClientError, ClientOSError,ContentTypeError
@@ -26,12 +27,13 @@ from asyncio import Lock
 from urllib.parse import urlencode
 from datetime import datetime, timedelta
 from apis.webull.webull_trading import WebullTrading
-from apis.y_finance.yf_sdk import yfSDK
+
 import numpy as np
 from .models.option_models.universal_snapshot import UniversalOptionSnapshot, UniversalSnapshot, OptionData, UniversalOptionSnapshot2
 from .polygon_helpers import flatten_nested_dict, flatten_dict
 lock = Lock()
 trading = WebullTrading()
+
 sema = asyncio.Semaphore(4)
 # Function to map Pandas dtypes to PostgreSQL types
 def dtype_to_postgres(dtype):
@@ -59,7 +61,6 @@ class PolygonOptions:
         self.connection_string = os.environ.get('POLYGON_STRING')
 
         self.session = None
-
         self.api_key = os.environ.get('YOUR_POLYGON_KEY')
         self.today = datetime.now().strftime('%Y-%m-%d')
         self.yesterday = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d')
@@ -72,8 +73,10 @@ class PolygonOptions:
         self.eight_days_ago = (datetime.now() - timedelta(days=8)).strftime('%Y-%m-%d')
         self.one_year_from_now = (datetime.now() + timedelta(days=365)).strftime('%Y-%m-%d')
         self.one_year_ago = (datetime.now() - timedelta(days=365)).strftime('%Y-%m-%d')
-        self.yf = yfSDK()
+     
 
+        self._45_days_from_now = (datetime.now() + timedelta(days=45)).strftime('%Y-%m-%d')
+        self._90_days_drom_now = (datetime.now() + timedelta(days=90)).strftime('%Y-%m-%d')
 
         self.db_config = {
             "host": host, # Default to this IP if 'DB_HOST' not found in environment variables
@@ -97,11 +100,42 @@ class PolygonOptions:
             await self.http_session.close()
             self.http_session = None
 
-    async def fetch_page(self, url):
-        # This is just a placeholder function. You need to define how to fetch each page.
-        async with aiohttp.ClientSession() as session:
-            async with session.get(url) as response:
-                return await response.json()
+    async def fetch_page(self, session, url):
+        if 'apiKey' not in url:
+            url = f"{url}?apiKey={os.environ.get('YOUR_POLYGON_KEY')}"
+        try:
+            response = await session.get(url)
+            return response.json()
+        except Exception as e:
+            print(f"ERROR: {e}")
+
+    async def paginate_concurrent(self, url, as_dataframe=False, concurrency=250, filter:bool=False):
+        all_results = []
+        pages_to_fetch = [url]
+
+        async with httpx.AsyncClient() as session:
+            while pages_to_fetch:
+                tasks = []
+                current_urls = pages_to_fetch[:concurrency]
+                pages_to_fetch = pages_to_fetch[concurrency:]  # Update remaining pages
+
+                for next_url in current_urls:
+                    tasks.append(self.fetch_page(session, next_url))
+
+                results = await asyncio.gather(*tasks)
+                for data in results:
+                    if data is not None and "results" in data:
+                        all_results.extend(data["results"])
+                        next_url = data.get("next_url")
+                        if next_url:
+                            next_url += f'&{urlencode({"apiKey": os.environ.get("YOUR_POLYGON_KEY")})}'
+                            pages_to_fetch.append(next_url)
+
+        if as_dataframe:
+            import pandas as pd
+            return pd.DataFrame(all_results)
+            
+        return all_results
     def sanitize_value(self, value, col_type):
         """Sanitize and format the value for SQL query."""
         if col_type == 'str':
@@ -127,11 +161,12 @@ class PolygonOptions:
 
         return self.pool
     async def update_options(self, ticker):
-        async with sema:
-            all_options = await self.find_symbols(ticker)
-            df = all_options.df
-            print(df)
-            await self.batch_insert_dataframe(df, table_name='poly_options', unique_columns='option_symbol')
+        all_options = await self.find_symbols(ticker)
+        df = all_options.df
+        print(df)
+        await self.batch_insert_dataframe(df, table_name='opts', unique_columns='option_symbol')
+
+
 
     
     async def filter_options(self,select_columns:str=None, **kwargs):
@@ -142,7 +177,7 @@ class PolygonOptions:
                                  expire_date='2023-01-01', delta_min=0.1, delta_max=0.5)
         """
         # Start with the base query
-        query = f"SELECT * FROM poly_options WHERE "
+        query = f"SELECT * FROM opts WHERE "
         params = []
         param_index = 1
 
@@ -278,7 +313,7 @@ class PolygonOptions:
         
      
         # Dynamically build query based on kwargs
-        query = f"SELECT * FROM poly_options WHERE oi > 0" if select_columns is None else f"SELECT ticker, strike, cp, expiry, {select_columns} FROM poly_options WHERE oi > 0"
+        query = f"SELECT * FROM opts WHERE oi > 0" if select_columns is None else f"SELECT ticker, strike, cp, expiry, {select_columns} FROM opts WHERE oi > 0"
 
         
 
@@ -308,7 +343,7 @@ class PolygonOptions:
             return []
     async def fetch_filtered_records(self, **kwargs):
         # Start with the base SELECT statement
-        select_sql = "SELECT ticker, strike, expiry, cp FROM poly_options"
+        select_sql = "SELECT ticker, strike, expiry, cp FROM opts"
 
         # Define which attributes can have min/max filters
         numeric_attrs = ['strike', 'dte', 'time_value', 'liquidity_score', 'theta', 
@@ -345,9 +380,14 @@ class PolygonOptions:
                 conditions.append(f"{column} = ${len(conditions) + 1}")
                 values.append(value)
 
+
+
         if conditions:
             select_sql += " WHERE " + " AND ".join(conditions)
 
+
+
+
         try:
             async with self.pool.acquire() as conn:
                 # Fetch records with the constructed query and values
@@ -364,7 +404,7 @@ class PolygonOptions:
 
     async def fetch_records(self):
         await self.connect()
-        select_sql = "SELECT * FROM poly_options"  # Modify this query based on your specific needs
+        select_sql = "SELECT * FROM opts"  # Modify this query based on your specific needs
         async with self.pool.acquire() as conn:
             # Fetch records
             records = await conn.fetch(select_sql)
@@ -380,9 +420,26 @@ class PolygonOptions:
             'datetime64': 'TIMESTAMP',
             'datetime64[ns]': 'timestamp',
             'datetime64[ms]': 'timestamp',
-            'datetime64[ns, US/Eastern]': 'TIMESTAMP WITH TIME ZONE'
+            'datetime64[ns, US/Eastern]': 'TIMESTAMP WITH TIME ZONE',
+            'string': 'TEXT',
+            'int32': 'INTEGER',
+            'float32': 'FLOAT',
+            'datetime64[us]': 'timestamp',
+            'timedelta[ns]': 'INTERVAL',
+            'category': 'TEXT',
+            'int16': 'SMALLINT',
+            'int8': 'SMALLINT',
+            'uint8': 'SMALLINT',
+            'uint16': 'INTEGER',
+            'uint32': 'BIGINT',
+            'uint64': 'NUMERIC',
+            'complex64': 'COMPLEX',
+            'complex128': 'COMPLEX',
+            'bytearray': 'BLOB',
+            'bytes': 'BLOB',
+            'memoryview': 'BLOB'
         }
-    
+            
         # Check for large integers and update dtype_mapping accordingly
         for col, dtype in zip(df.columns, df.dtypes):
             if dtype == 'int64':
@@ -666,24 +723,20 @@ class PolygonOptions:
                     for record in data:
                         new_record = []
                         for col, val in zip(df.columns, record):
-                            if col == 'insertion_timestamp':
-                                val = pd.Timestamp(val).to_pydatetime()
-                                        
                             pg_type = type_mapping[col]
 
                             if val is None:
                                 new_record.append(None)
-                            elif pg_type == 'timestamp' and isinstance(val, np.datetime64):
-                                new_record.append(pd.Timestamp(val).to_pydatetime().replace(tzinfo=None))
-
-            
-                            elif isinstance(val, datetime):
-                                new_record.append(pd.Timestamp(val).to_pydatetime())
-                            elif pg_type in ['timestamp', 'timestamp without time zone', 'timestamp with time zone'] and isinstance(val, np.datetime64):
-                                new_record.append(pd.Timestamp(val).to_pydatetime().replace(tzinfo=None))  # Modified line
+                            elif pg_type in ['timestamp', 'timestamp without time zone', 'timestamp with time zone']:
+                                if isinstance(val, np.datetime64):
+                                    # Convert numpy datetime64 to Python datetime, ensure UTC and remove tzinfo if needed
+                                    new_record.append(pd.Timestamp(val).to_pydatetime().replace(tzinfo=None))
+                                elif isinstance(val, datetime):
+                                    # Directly use the Python datetime object
+                                    new_record.append(val)
                             elif pg_type in ['double precision', 'real'] and not isinstance(val, str):
                                 new_record.append(float(val))
-                            elif isinstance(val, np.int64):  # Add this line to handle numpy.int64
+                            elif isinstance(val, np.int64):
                                 new_record.append(int(val))
                             elif pg_type == 'integer' and not isinstance(val, int):
                                 new_record.append(int(val))
@@ -701,18 +754,17 @@ class PolygonOptions:
                             except Exception as e:
                                 print(f"An error occurred while inserting the record: {e}")
                                 await connection.execute('ROLLBACK')
-                                raise
-
-                    if batch_data:  # Don't forget the last batch
-        
+                    
+                    if batch_data:
+                        # Convert all elements in batch_data to strings
+                        #batch_data = [[str(value) if isinstance(value, float) else value for value in row] for row in batch_data]
+                        
                         try:
-               
-
                             await connection.executemany(insert_query, batch_data)
                         except Exception as e:
                             print(f"An error occurred while inserting the record: {e}")
                             await connection.execute('ROLLBACK')
-                            raise
+                            
     async def save_to_history(self, df, main_table_name, history_table_name):
         # Assume the DataFrame `df` contains the records to be archived
         if not await self.table_exists(history_table_name):
@@ -841,70 +893,9 @@ class PolygonOptions:
         datas = await trading.stock_quote(ticker)
 
         price = datas.web_stock_close
-
-        return price
-    async def get_option_strikes_by_moneyness(self, ticker, current_price, expiration_date):
-        """
-        Get option strikes categorized by moneyness and term.
-
-        :param ticker: The ticker symbol of the underlying asset.
-        :param current_price: Current price of the underlying asset.
-        :return: Dictionary of options categorized by moneyness and term.
-        """
-        all_data = await self.get_option_chain_all(ticker, expiration_date=expiration_date)
-
-        # Define your categorization thresholds for ITM, Deep ITM, ATM, OTM, Deep OTM
-        deep_itm_threshold = 0.2  # e.g., 20% ITM is considered Deep ITM
-        atm_threshold = 0.05      # e.g., within 5% of current price is considered ATM
-        deep_otm_threshold = 0.2  # e.g., 20% OTM is considered Deep OTM
-
-        # Define expiry terms (assuming all_data.df contains a 'expiry' column in YYYY-MM-DD format)
-        today = datetime.today().date()
-        one_month = today + timedelta(days=30)
-        three_months = today + timedelta(days=90)
-
-        near_term = lambda expiry_date: today <= expiry_date <= one_month
-        mid_term = lambda expiry_date: one_month < expiry_date <= three_months
-        far_term = lambda expiry_date: expiry_date > three_months
-
-        categorized_strikes = {
-            'Near Term': {'ITM': [], 'Deep ITM': [], 'ATM': [], 'OTM': [], 'Deep OTM': []},
-            'Mid Term': {'ITM': [], 'Deep ITM': [], 'ATM': [], 'OTM': [], 'Deep OTM': []},
-            'Far Term': {'ITM': [], 'Deep ITM': [], 'ATM': [], 'OTM': [], 'Deep OTM': []},
-        }
-
-        # Iterate through the options data and categorize based on moneyness and term
-        for option in all_data.df.itertuples():
-            # Calculate moneyness
-            moneyness = (option.strike - current_price) / current_price
-            expiry_date = datetime.strptime(option.expiry, '%Y-%m-%d').date()
-
-            term_category = None
-            if near_term(expiry_date):
-                term_category = 'Near Term'
-            elif mid_term(expiry_date):
-                term_category = 'Mid Term'
-            elif far_term(expiry_date):
-                term_category = 'Far Term'
-
-            moneyness_category = None
-            if moneyness <= -deep_itm_threshold:
-                moneyness_category = 'Deep ITM'
-            elif -deep_itm_threshold < moneyness <= -atm_threshold:
-                moneyness_category = 'ITM'
-            elif -atm_threshold < moneyness < atm_threshold:
-                moneyness_category = 'ATM'
-            elif atm_threshold <= moneyness < deep_otm_threshold:
-                moneyness_category = 'OTM'
-            elif moneyness >= deep_otm_threshold:
-                moneyness_category = 'Deep OTM'
-
-
-            if term_category and moneyness_category:
-                # Append a tuple of (option, moneyness_category) to the categorized strikes
-                categorized_strikes[term_category][moneyness_category].append((option, moneyness_category))
-
-        return categorized_strikes
+        if price is not None:
+            return float(price[0])
+    
     async def get_option_contracts(self, ticker:str, 
                                    strike_price_greater_than:str=None, 
                                    strike_price_less_than:str=None, 
@@ -999,7 +990,32 @@ class PolygonOptions:
                             return atm_tickers
                         else:
                             return None
-                        
+
+
+    async def aggregate_volume_oi(self, ticker, start_date:str=None, end_date:str=None):
+        if start_date == None:
+            start_date = self._45_days_from_now
+
+        if end_date == None:
+            end_date = self._90_days_drom_now
+
+        all_options = await self.get_near_the_money_data(ticker, exp_less_than=end_date, exp_greater_than=start_date)
+
+        if all_options is not None:
+            
+            # Group by 'call_put' and then 'expiry'
+            grouped = all_options.groupby(['cp', 'expiry'])
+            print(grouped)
+
+            aggregated_data = grouped.agg({
+                'vol': 'sum',
+                'strike': 'mean',
+                'oi': 'sum',
+
+            })
+                    
+
+            return aggregated_data
     async def get_near_the_money_data(self, ticker: str, exp_greater_than:str='2023-11-17', exp_less_than:str='2023-12-15'):
         """
         Gets options near the money for a ticker
@@ -1069,7 +1085,7 @@ class PolygonOptions:
                 pass
 
 
-    async def get_option_chain_all(self, underlying_asset, strike_price=None, strike_price_lte=None, strike_price_gte=None, expiration_date=None, expiration_date_gte=None, expiration_date_lite=None, contract_type=None, order=None, limit=250, sort=None, pool=None):
+    async def get_option_chain_all(self, underlying_asset, strike_price=None, strike_price_lte=None, strike_price_gte=None, expiration_date=None, expiration_date_gte=None, expiration_date_lite=None, contract_type=None, order=None, limit=250, sort=None, pool=None, insert:bool=False):
         """
         Get all options contracts for an underlying ticker across all pages.
 
@@ -1170,7 +1186,7 @@ class PolygonOptions:
                                           change_to_breakeven, underlying_price, ticker, return_on_risk, 
                                           velocity):
         insert_sql = """
-        INSERT INTO poly_opts (
+        INSERT INTO opts (
             strike, expiry, dte, time_value, moneyness, liquidity_score, cp, 
             exercise_style, option_symbol, theta, theta_decay_rate, delta, 
             delta_theta_ratio, gamma, gamma_risk, vega, vega_impact, timestamp, 
@@ -1615,19 +1631,51 @@ class PolygonOptions:
         # Fetch records
 
         return chunks
-    async def multi_snapshot(self, symbol_string:str):
-
-        async with aiohttp.ClientSession() as session:
-            url=f"https://api.polygon.io/v3/snapshot?ticker.any_of={symbol_string}&apiKey={os.environ.get('YOUR_POLYGON_KEY')}&limit=250"
-            async with session.get(url) as resp:
-                data = await resp.json()
-                results = data['results'] if 'results' in data else None
-                if results is not None:
-                    data = OptionData(results)
-
-
-                    return data
+    async def multi_snapshot_httpx(self, symbol_string: str):
+        async with httpx.AsyncClient() as client:
+            url = f"https://api.polygon.io/v3/snapshot?ticker.any_of={symbol_string}&apiKey={os.environ.get('YOUR_POLYGON_KEY')}&limit=250"
+            resp = await client.get(url)
+            data = resp.json()
+            results = data['results'] if 'results' in data else None
+            if results is not None:
+                data = OptionData(results)
+                return data
     async def close(self):
         if self.pool:
             await self.pool.close()
             self.pool = None
+
+
+
+    async def all_options_snapshot(self):
+        endpoint = f"https://api.polygon.io/v3/snapshot?type=options&apiKey={self.api_key}&limit=250"
+
+
+        data = await self.paginate_concurrent(endpoint)
+        print(data)
+        data = data['results'] if 'results' in data else None
+        break_even_price = [i.get('break_even_price') for i in data]
+        session = [i.get('session') for i in data]
+        details = [i.get('details') for i in data]
+        greeks = [i.get('greeks') for i in data]
+        delta = [i.get('delta') for i in greeks]
+        gamma = [i.get('gamma') for i in greeks]
+        theta = [i.get('theta') for i in greeks]
+        vega = [i.get('vega') for i in greeks]
+        last_quote = [i.get('last_quote') for i in data]
+        ask = [i.get('ask') for i in last_quote]
+        bid = [i.get('bid') for i in last_quote]
+        ask_ex = [i.get('ask_exchange') for i in last_quote]
+        bid_ex = [i.get('bid_exchange') for i in last_quote]
+        bid_size = [i.get('bid_size') for i in last_quote]
+        ask_size = [i.get('ask_size') for i in last_quote]
+        
+        last_trade = [i.get('last_trade') for i in data]
+        open_interest = [i.get('open_interest') for i in data]
+        underlying_asset = [i.get('underlying_asset') for i in data]
+        name = [i.get('name') for i in data]
+        market_status = [i.get('market_status') for i in data]
+        ticker = [i.get('ticker') for i in data]
+        type = [i.get('type') for i in data]
+        print(underlying_asset)
+        print(details)
