@@ -433,8 +433,10 @@ class DatabaseManager:
             #print("Initial DataFrame columns:", df.columns.tolist())
             
             df = df.copy()
-            df.dropna(inplace=True)
-            df['insertion_timestamp'] = [datetime.now() for _ in range(len(df))]
+            try:
+                df['insertion_timestamp'] = [datetime.now() for _ in range(len(df))]
+            except Exception as e:
+                df['insertion_timestamp'] = None
 
             # Debug: Print DataFrame columns after modifications
             #print("Modified DataFrame columns:", df.columns.tolist())
