@@ -7,6 +7,7 @@ import plotly.graph_objects as go
 from selenium import webdriver
 from selenium.webdriver.chrome.options import Options
 import aiohttp
+
 from plots import plot_calls_and_puts,plot_greek_exposure,plot_iv_surface
 from disnake.ext import commands
 from discord_.bot_menus.pagination import AlertMenus, PageSelect
@@ -14,6 +15,7 @@ from discord_.autocomp import ticker_autocomp,strike_autocomp,expiry_autocomp
 from apis.polygonio.polygon_options import PolygonOptions
 from apis.webull.webull_options import WebullOptions
 from apis.webull.webull_trading import WebullTrading
+from apis._openai.openai_sdk import OpenAISDK
 from apis.polygonio.polygon_database import PolygonDatabase
 from tabulate import tabulate
 from all_helpers import determine_emoji
@@ -29,7 +31,7 @@ class OptionsCOG(commands.Cog):
         self.wt = WebullTrading()
         self.wb = WebullOptions(user='chuck', database='markets', host='localhost', password='fud', port=5432)
         self.db = PolygonDatabase(user='chuck', database='markets', host='localhost', password='fud', port=5432)
-
+        self.ai = OpenAISDK()
 
 
     @commands.slash_command()
@@ -310,29 +312,6 @@ class OptionsCOG(commands.Cog):
         await inter.edit_original_message(embed=embeds[0], view=AlertMenus(embeds))
 
 
-    @options.sub_command()
-    async def all_trades(self, inter:disnake.AppCmdInter, ticker:str=commands.Param(autocomplete=ticker_autocomp)):
-        """View all option trades for an underlying ticker - ordered by volume"""
-        await self.poly.connect()
-        await inter.response.defer()
-
-        query = f"""SELECT underlying_symbol, strike, call_put, expiry,size, price FROM optiontrade WHERE underlying_symbol = '{ticker}' and expiry > '2023-10-04' ORDER BY size DESC LIMIT 5000"""
-
-        results = await self.poly.fetch(query)
-
-
-
-        df = pd.DataFrame(results, columns=['sym', 'strike', 'call/put', 'exp', 'size', 'price'])
-        table = tabulate(df, headers='keys', tablefmt='fancy', showindex=False)
-
-        chunks = [table[i:i + 4000] for i in range(0, len(table), 4000)]
-        embeds=[]
-        for chunk in chunks:
-            embed = disnake.Embed(title=f"All Trades: {ticker}", description=f"```py\n{chunk}```", color=disnake.Colour.dark_purple())
-            embed.add_field(name=f"Viewing all option trades for:", value=f"> **{ticker}**")
-            embeds.append(embed)
-
-        await inter.edit_original_message(embed=embeds[0], view=AlertMenus(embeds))
 
 
     @options.sub_command()
@@ -461,6 +440,20 @@ class OptionsCOG(commands.Cog):
         await inter.edit_original_message(file=file, embed=embed, view = OptionsView(ticker).add_item(OptsSelect(ticker)))
 
     
+     
+    @options.sub_command()
+    async def filter(self, inter:disnake.AppCmdInter, filter:str):
+        """Filter options!"""
+        await inter.response.defer()
+        ai = await self.ai.run_conversation(f"Hey, can you look at some options for me? I want: {filter}.")
+        
+        embed = disnake.Embed(title=f"Filter Options", description=f"```py\n{ai}```")
+
+
+        await inter.edit_original_message(embed=embed)
+
+
+
 
 
 opts = OptionsCOG(commands.Bot)
